% IISPH for Wiki dataset

clear all; clc;
load wikiData;
fprintf('Wiki dataset loaded...\n');

%% parameter settings
run = 9;
map = zeros(run,2);
alphas = [0.5 0.5];
beta = 1e-3;
mu = 1e-2;
lambda = 1e-2;
gamma = 0.00001;
bits = 128;
k = 10;

fprintf('parameters:\n code length = %d\n mu = %d\n gamma = %.4f\n', bits, mu, gamma);

%% centralization
fprintf('centralizing data...\n');

I_te = bsxfun(@minus, I_te, mean(I_tr, 1));
I_tr = bsxfun(@minus, I_tr, mean(I_tr, 1));
T_te = bsxfun(@minus, T_te, mean(T_tr, 1));
T_tr = bsxfun(@minus, T_tr, mean(T_tr, 1));


I_te = I_te';
I_tr = I_tr';
T_te = T_te';
T_tr = T_tr';


%% compute inter-modality similarity lapasian matrix
m = size(L_tr,1);
A12 = zeros(m,m);
for i = 1:m
    similar_ins = find(L_tr == L_tr(i));
    A12(i,similar_ins)=1;
end

D12 = diag(sum(A12));

%%  iteration
for i = 1 : run
tic


fprintf('\n\n');
fprintf('run %d starts...\n', i);
I_temp = I_tr;
T_temp = T_tr;
[row, col]= size(I_temp);
[rowt, colt] = size(T_temp);


I_temp = bsxfun(@minus,I_temp , mean(I_temp,2));
T_temp = bsxfun(@minus,T_temp, mean(T_temp,2));
Im_te = (bsxfun(@minus, I_te, mean(I_tr, 2)));
Te_te = (bsxfun(@minus, T_te, mean(T_tr, 2)));

%% solve objective function
fprintf('start solving IISPH...\n');
[U1, U2, P1, P2, V] = solveIISPH( I_temp, T_temp, A12, D12, k, alphas, beta, mu, lambda, gamma, bits);

%% extend to the whole database
V_total =( alphas(1) * U1' * U1 + alphas(2) * U2' * U2 + beta * eye(bits) + beta * eye(bits) + gamma * eye(bits) ) \ ( alphas(1) * U1' * I_tr + alphas(2) * U2' * T_tr + beta * P1 * I_tr + beta * P2 * T_tr);
  
%% calculate hash codes
Vi_tr = sign((bsxfun(@minus, V_total , mean(V,2))));
Vi_te = sign((bsxfun(@minus, P1*I_te , mean(V,2))));
Vt_tr = sign((bsxfun(@minus, V_total , mean(V,2))));
Vt_te = sign((bsxfun(@minus, P2*T_te , mean(V,2))));

%% evaluate
fprintf('start evaluating...\n');
simti = Vi_tr' * Vt_te;    % Text Query
simit = Vt_tr' * Vi_te;    % Image Query
map(i, 1) = mAP(simti,L_tr,L_te);
map(i, 2) = mAP(simit,L_tr,L_te);

fprintf('mAP at run %d runs for TextQueryOnImageDB: %.4f\n', i, map(i, 1));
fprintf('mAP at run %d runs for ImageQueryOnTextDB: %.4f\n', i, map(i, 2));
toc

end
mean(map);
fprintf('\nbits = %d, mu = %d, gamma = %.4f\n', bits, mu, gamma);
fprintf('average map over %d runs for TextQueryOnImageDB: %.4f\n', run, mean(map( : , 1)));
fprintf('average map over %d runs for ImageQueryOnTextDB: %.4f\n', run, mean(map( : , 2)));


