function [L, D, W] = calc_Laplacian(Dist, k)

%%
% 
% input: Dist, m*m, 
%        k k-nearest neighbors
% m: the number of the instances

% output: W, D, L

%%
m = size(Dist,1);

W = zeros(m,m);

% median value of distance matrix
median_D = median(Dist(:));

% similarity matrix
Affinity  = exp(-Dist.^2/median_D);    % Attention, wether needs ^2 


for i = 1:m
    Dist(i,i) = inf;
end

% collect its k-nearest neighbors  k*m matrix
[dists, klist] = knn(Dist,k);

% compute the incidence matrix H
for i=1:m
    W(klist(:,i),i) = Affinity(klist(:,i),i);
end


for i=1:m
    for j=i:m
        if W(i,j) ~= W(j,i)
            if(W(i,j) == 0)
                W(i,j) = W(j,i);
            else
                W(j,i) = W(i,j);
            end
        end
    end
end


% compute hyperedge weight
Dii = sum(W);

% compute weight matrix
D = diag(Dii);

L = D - W;

%%